/**
 * Gamecube controller to Nintendo 64 adapter
 * by Andrew Brown
 * Rewritten for N64 to HID by Peter Den Hartog
 * Then Rewritten once again for USB Gamepad compliance by Bob Jones
 */

/**
 * To use, hook up the following to the Arduino Leonardo:
 * Digital I/O 2: N64 serial line
 * All appropriate grounding and power lines
 */
 
#include "crc_table.h"
#include "N64_Controller.h"

//Debug mode
//#define _DEBUG_ON

// Controller Buttons
#define N64_A        8192  // 0010000000000000
#define N64_B        4096  // 0001000000000000
#define N64_Z        2048  // 0000100000000000
#define N64_START    1024  // 0000010000000000
#define N64_DUP      512   // 0000001000000000
#define N64_DDOWN    256   // 0000000100000000
#define N64_DLEFT    128   // 0000000010000000
#define N64_DRIGHT   64    // 0000000001000000
#define N64_L        32    // 0000000000100000
#define N64_R        16    // 0000000000010000
#define N64_CUP      8     // 0000000000001000
#define N64_CDOWN    4     // 0000000000000100
#define N64_CLEFT    2     // 0000000000000010
#define N64_CRIGHT   1     // 0000000000000001

// Joystick State
JoyState_t JoySt;

N64_Controller N64Controller;

void setup()
{
#ifdef _DEBUG_ON
  Serial.begin(38400);

  //Needed for Leonardo
  while(!Serial);
#endif

  N64Controller.Initialize();
  
  JoySt.XAxis = 127;
  JoySt.YAxis = 127;
  JoySt.Buttons = 0;
}

void loop()
{
  N64Controller.Update();
  
  JoySt.XAxis = 127 + N64Controller.GetStick_x();
  JoySt.YAxis = 127 + N64Controller.GetStick_y();
  JoySt.Buttons = N64Controller.Getbuttons();

#ifdef _DEBUG_ON
  Serial.println((uint16_t)JoySt.Buttons, BIN);
#endif
  
  Joystick.setState(&JoySt);
  delay(25);
}
